package hwr;

import hwr.featurizer.DimensionRate;
import hwr.featurizer.Featurizer;
import hwr.featurizer.GridPixelRateFeature;
import hwr.featurizer.HMirrorFeature;
import hwr.featurizer.HistogramFeature;
import hwr.featurizer.VMirrorFeature;
import hwr.normalizer.Normalizer;
import hwr.normalizer.RectangleNormalizer;
import hwr.recognizer.BestManhattanDistanceRecognizer;
import hwr.recognizer.Best_ManhattanDistanceRecognizer;
import hwr.recognizer.LimitedBest_ManhattanDistanceRecognizer;
import hwr.recognizer.Recognizer;
import java.util.TreeMap;
import java.util.ArrayList;
import java.util.Date;

public class Launch implements Runnable {

    private static String dbTestFileName;
    private static TreeMap<Character, ArrayList<BitmapCharacter>> mbCharMap;
    private static TreeMap<Character, ArrayList<ArrayList<Double>>> trainFeatureMap;
    private static TreeMap<Character, ArrayList<ArrayList<Double>>> testFeatureMap;
    private static GUI gui;

    public static void main(String[] args) {
        gui = new GUI();
        for (int i = 0; i < args.length; i++) {
            if (i == args.length - 1) {
                gui.txtTestDatabase.setText(args[i]);
            } else {
                gui.dbListModel.addElement(args[i]);
            }
        }
        new Thread(gui).start();
    }

    public static void runTest() {
        new Thread(new Launch()).start();
    }

    public void run() {

        dbTestFileName = gui.txtTestDatabase.getText();

        mbCharMap = new TreeMap<Character, ArrayList<BitmapCharacter>>();

        long startTime = System.currentTimeMillis();

        gui.txtOutput.append("Test start at " + new Date(startTime).toString() + "\n");

        gui.txtOutput.append("Building train database...");
        for (int i = 0; i < gui.lstTrainDatabases.getModel().getSize(); i++) {
            String dbFileName = gui.lstTrainDatabases.getModel().getElementAt(i).toString();
            TreeMap<Character, ArrayList<BitmapCharacter>> part_mbCharMap = BitmapCharDBReader.getCharTreeMap(dbFileName);
            for (Character key : part_mbCharMap.keySet()) {
                {
                    if (mbCharMap.containsKey(key)) {
                        ArrayList<BitmapCharacter> mbCharList = mbCharMap.get(key);
                        mbCharList.addAll(part_mbCharMap.get(key));
                    } else {
                        mbCharMap.put(key, part_mbCharMap.get(key));
                    }
                }
            }
        }
        gui.txtOutput.append(" Done!" + "\n");

        int ROWS;
        int COLS;
        double WEIGHT;
        WEIGHT = 3;
        Featurizer.addModule(new HMirrorFeature(WEIGHT));
        Featurizer.addModule(new VMirrorFeature(WEIGHT));
        Featurizer.addModule(new DimensionRate(WEIGHT));

        // Upper Lower profile
        ROWS = 2;
        COLS = 10;
        for (int i = 0; i < ROWS; i++) {
            for (int j = 0; j < COLS; j++) {
                Featurizer.addModule(new GridPixelRateFeature(ROWS, COLS, i, j));
            }
        }
        // Left right profile
        ROWS = 10;
        COLS = 2;
        for (int i = 0; i < ROWS; i++) {
            for (int j = 0; j < COLS; j++) {
                Featurizer.addModule(new GridPixelRateFeature(ROWS, COLS, i, j));
            }
        }
        //
        ROWS = 3;
        COLS = 3;
        for (int i = 0; i < ROWS; i++) {
            for (int j = 0; j < COLS; j++) {
                Featurizer.addModule(new GridPixelRateFeature(ROWS, COLS, i, j));
            }
        }
        //
        ROWS = 5;
        COLS = 5;
        for (int i = 0; i < ROWS; i++) {
            for (int j = 0; j < COLS; j++) {
                Featurizer.addModule(new GridPixelRateFeature(ROWS, COLS, i, j));
            }
        }
        // Histograms
        int BARS = 10;
        WEIGHT = 4;
        for (int barIndex = 0; barIndex < BARS; barIndex++) {
            Featurizer.addModule(new HistogramFeature(HistogramFeature.LEFT, BARS, barIndex, WEIGHT));
            Featurizer.addModule(new HistogramFeature(HistogramFeature.LOWER, BARS, barIndex, WEIGHT));
            Featurizer.addModule(new HistogramFeature(HistogramFeature.RIGHT, BARS, barIndex, WEIGHT));
            Featurizer.addModule(new HistogramFeature(HistogramFeature.UPPER, BARS, barIndex, WEIGHT));
        }

        Normalizer.addModule(new RectangleNormalizer());
        if (gui.chbUseNormalizer.isSelected()) {
            gui.txtOutput.append("Normalizing train database...");
            Normalizer.normalize(mbCharMap);
            gui.txtOutput.append(" Done!" + "\n");
        }

        gui.txtOutput.append("Featurizing train database...");
        trainFeatureMap = Featurizer.featurize(mbCharMap);
        gui.txtOutput.append(" Done!" + "\n");

        String module;
        if (gui.cmbRecognizer.getSelectedItem().equals("Best_ManhattanDistanceRecognizer")) {
            Recognizer.setModule(new Best_ManhattanDistanceRecognizer(Integer.parseInt(gui.txtMinNearestCharCount.getText())));
            module = "Best_ManhattanDistanceRecognizer(" + gui.txtMinNearestCharCount.getText() + ")";
        } else if (gui.cmbRecognizer.getSelectedItem().equals("LimitedBest_ManhattanDistanceRecognizer")) {
            Recognizer.setModule(new LimitedBest_ManhattanDistanceRecognizer(Integer.parseInt(gui.txtMinNearestCharCount.getText()),
                    Double.parseDouble(gui.txtCharAreaLimit.getText())));
            module = "LimitedBest_ManhattanDistanceRecognizer(" + gui.txtMinNearestCharCount.getText() + "," +
                    gui.txtCharAreaLimit.getText() + ")";
        } else {
            Recognizer.setModule(new BestManhattanDistanceRecognizer());
            module = "BestManhattanDistanceRecognizer";
        }
        gui.txtOutput.append("Recognizer: " + module + "\n");

        gui.txtOutput.append("Learning train database...");
        Recognizer.learn(trainFeatureMap);
        gui.txtOutput.append(" Done!" + "\n");

        gui.txtOutput.append("Building test database...");
        mbCharMap = BitmapCharDBReader.getCharTreeMap(dbTestFileName);
        gui.txtOutput.append(" Done!" + "\n");

        if (gui.chbUseNormalizer.isSelected()) {
            gui.txtOutput.append("Normalizing test database...");
            Normalizer.normalize(mbCharMap);
            gui.txtOutput.append(" Done!" + "\n");
        }

        gui.txtOutput.append("Featurizing test database...");
        testFeatureMap = Featurizer.featurize(mbCharMap);
        gui.txtOutput.append(" Done!" + "\n");

        int totalTestCharCount = 0;
        int totalProgressCounter = 1;
        for (Character key : testFeatureMap.keySet()) {
            totalTestCharCount += testFeatureMap.get(key).size();
        }
        gui.lblTestDatabaseSize.setText(Integer.toString(totalTestCharCount));
        gui.prgTotal.setMaximum(totalTestCharCount);
        gui.prgTotal.setValue(0);

        int totalRecognized = 0;
        int totalFailed = 0;

        gui.txtOutput.append("Recognizing..." + "\n");

        for (Character key : testFeatureMap.keySet()) {

            int currentCharCount = testFeatureMap.get(key).size();

            gui.lblCurrentCharacter.setText(key.toString());
            gui.lblCurrentDatabaseSize.setText(Integer.toString(currentCharCount));
            gui.prgCurrent.setMaximum((int) currentCharCount);
            gui.prgCurrent.setValue(0);
            int currentProgressCounter = 1;
            int currentRecognized = 0;
            int currentFailed = 0;

            String strRecognizerOutput = " ";

            ArrayList<ArrayList<Double>> charFeatArrayList = testFeatureMap.get(key);
            for (ArrayList<Double> vect : charFeatArrayList) {

                char c = Recognizer.recognize(vect);
                strRecognizerOutput += c + " ";

                if (Character.toUpperCase(c) == Character.toUpperCase(key)) {

//                    if (c == key) {
                    totalRecognized++;
                    currentRecognized++;
                    gui.lblRecognizedCharacters.setText(Integer.toString(totalRecognized));
                    gui.lblCurrentRecognizedCharacters.setText(Integer.toString(currentRecognized));
                    gui.lblCurrentAccuracy.setText(Double.toString((double) currentRecognized / (double) currentProgressCounter));
                    gui.lblTotalAccuracy.setText(Double.toString((double) totalRecognized / (double) totalProgressCounter));
                } else {
                    totalFailed++;
                    currentFailed++;
                    gui.lblFailedCharacters.setText(Integer.toString(totalFailed));
                    gui.lblCurrentFailedCharacters.setText(Integer.toString(currentFailed));
                }

                gui.lblCurrentTestedCharacters.setText(Integer.toString(currentProgressCounter));
                gui.lblTotalTestedCharacters.setText(Integer.toString(totalProgressCounter));
                gui.prgCurrent.setValue(currentProgressCounter++);
                gui.prgTotal.setValue(totalProgressCounter++);

            }

            String strStatistic = key + " # " + Integer.toString(currentCharCount) + "/" +
                    Integer.toString(currentRecognized) + " = " +
                    Double.toString((double) currentRecognized / (double) currentCharCount);

            gui.txtOutput.append(strStatistic + "\n");

            System.out.println(strStatistic + " # " + strRecognizerOutput);
        }

        String strStatistic = "--------------------------------------------------------" + "\n" +
                "Tested character count: " + Integer.toString(totalTestCharCount) + "\n" +
                "Recognized character count: " + Integer.toString(totalRecognized) + "\n" +
                "Recognization rate: " + Double.toString(((double) totalRecognized / (double) totalTestCharCount) * 100) + " %";
        System.out.println(strStatistic);
        gui.txtOutput.append(strStatistic + "\n");

        long stopTime = System.currentTimeMillis();

        gui.txtOutput.append("Test done at " + new Date(stopTime).toString() + "\n");

        long totalTime = stopTime - startTime;
        int hours = (int) (totalTime / (60 * 60 * 1000));
        totalTime %= 60 * 60 * 1000;
        int minutes = (int) (totalTime / (60 * 1000));
        totalTime %= 60 * 1000;
        int seconds = (int) (totalTime / 1000);

        gui.txtOutput.append("Total time" + Integer.toString(hours) + ":" +
                Integer.toString(minutes) + ":" +
                Integer.toString(seconds) + "\n");
    }
}
