package hwr.featurizer;

import hwr.BitmapCharacter;

public class HistogramFeature implements FeaturizerModule {

    public static final int LEFT = 0;
    public static final int RIGHT = 1;
    public static final int UPPER = 2;
    public static final int LOWER = 3;
    private double weight;
    private int side,  barCount,  barIndex;

    public HistogramFeature(int side, int barCount, int barIndex, double weight) {
        this.side = side;
        this.barCount = barCount;
        this.barIndex = barIndex;
        this.weight = weight;
    }

    public double featurize(BitmapCharacter bmChar) {

        int width = bmChar.getWidth();
        int height = bmChar.getHeight();
        byte[][] charMatrix = bmChar.getData();

        int barSize = 0;
        int[] firstCharPixelIndex = null;
        int total = 0;

        switch (side) {
            case LEFT:
                barSize = height / barCount;
                firstCharPixelIndex = new int[barSize];
                for (int row = 0; row < barSize; row++) {
                    for (int col = 0; col < width; col++) {
                        if (charMatrix[(barIndex * barSize) + row][col] == 1) {
                            firstCharPixelIndex[row] = col;
                            break;
                        }
                    }
                }
                break;
            case RIGHT:
                barSize = height / barCount;
                firstCharPixelIndex = new int[barSize];
                for (int row = 0; row < barSize; row++) {
                    for (int col = 0; col < width; col++) {
                        if (charMatrix[(barIndex * barSize) + row][width - (col + 1)] == 1) {
                            firstCharPixelIndex[row] = col;
                            break;
                        }
                    }
                }
                break;
            case UPPER:
                barSize = width / barCount;
                firstCharPixelIndex = new int[barSize];
                for (int col = 0; col < barSize; col++) {
                    for (int row = 0; row < height; row++) {
                        if (charMatrix[row][(barIndex * barSize) + col] == 1) {
                            firstCharPixelIndex[col] = row;
                            break;
                        }
                    }
                }
                break;
            case LOWER:
                barSize = width / barCount;
                firstCharPixelIndex = new int[barSize];
                for (int col = 0; col < barSize; col++) {
                    for (int row = 0; row < height; row++) {
                        if (charMatrix[height - (row + 1)][(barIndex * barSize) + col] == 1) {
                            firstCharPixelIndex[col] = row;
                            break;
                        }
                    }
                }
                break;
            default:
                return 0;
        }

        double average;
        for (int subBar = 0; subBar < barSize; subBar++) {
            total += firstCharPixelIndex[subBar];
        }
        average = (double) total / (double) barSize;
        if (side == LEFT || side == RIGHT) {
            return average / (double) width;
        } else {
            return average / (double) height;
        }
    }
}
