package hwr.recognizer;

import java.util.ArrayList;
import java.util.TreeMap;
import java.util.ArrayList;

public class LimitedBest_ManhattanDistanceRecognizer implements RecognizerModule {

    TreeMap<Character, ArrayList<ArrayList<Double>>> featMap;
    private int count;
    private double limit;

    public LimitedBest_ManhattanDistanceRecognizer(int count, double limit) {
        this.count = count;
        this.limit = limit;
    }

    public void learn(TreeMap<Character, ArrayList<ArrayList<Double>>> featMap) {
        this.featMap = featMap;
    }

    public Character recognize(ArrayList<Double> featArrayList) {

        ArrayList<Character> charArray = new ArrayList<Character>();
        ArrayList<Double> distArray = new ArrayList<Double>();
        ArrayList<ArrayList<Double>> charFeatArrayList;
        TreeMap<Character, Integer> charMap = new TreeMap<Character, Integer>();
        double manhattanDistance;
        double bestManhattanDistance;
        int bestIndex;
        int featureSize = featMap.get(featMap.firstKey()).get(0).size();
        double charDistanceLimit = (double) featureSize * limit / (double) 100;

        for (Character key : featMap.keySet()) {
            charMap.put(key, 0);
            charFeatArrayList = featMap.get(key);
            for (ArrayList<Double> sampleArrayList : charFeatArrayList) {
                manhattanDistance = 0;
                for (int i = 0; i < featureSize; i++) {
                    manhattanDistance += Math.abs(sampleArrayList.get(i) - featArrayList.get(i));
                }
                if (manhattanDistance <= charDistanceLimit) {
                    return key;
                }
                charArray.add(key);
                distArray.add(manhattanDistance);
            }
        }

        while (true) {
            bestManhattanDistance = Double.MAX_VALUE;
            bestIndex = 0;
            for (int i = 0; i < distArray.size(); i++) {
                if (distArray.get(i) < bestManhattanDistance) {
                    bestManhattanDistance = distArray.get(i);
                    bestIndex = i;
                }
            }
            charMap.put(charArray.get(bestIndex), charMap.get(charArray.get(bestIndex)) + 1);
            if (charMap.get(charArray.get(bestIndex)) == count) {
                return charArray.get(bestIndex);
            }
            charArray.remove(bestIndex);
            distArray.remove(bestIndex);
        }
    }
}

